import { Injectable } from '@angular/core';
import Localbase from 'localbase';
import { Router } from '@angular/router';
import { BehaviorSubject } from 'rxjs';
import { Attribute } from 'src/interfaces/attribute';
import { Skill } from 'src/interfaces/skill';
import { Weapon } from 'src/interfaces/weapon';
import { Spell } from 'src/interfaces/spell';
import { Ability } from 'src/interfaces/ability';
import { Trait } from 'src/interfaces/traits';
import { SimpleItem } from 'src/interfaces/simple-item';
import { Food } from 'src/interfaces/food';

@Injectable({
  providedIn: 'root',
})
export class DataService {
  private db: any;

  public dataLoaded = false;
  public characterName: string = '';

  constructor(private Router: Router) {
    this.db = new Localbase('DNDTools');
    // this.db.config.debug = false;
  }

  async loadData(): Promise<any> {
    console.warn('loadData() called');
    this.characterName = sessionStorage.getItem('characterName')!;
    if (this.dataLoaded) {
      return Promise.resolve();
    }
    return this.getCollectionWithKeys(this.characterName).then((data: any) => {
      this.buildCurrentCharacter(data);
      this.dataLoaded = true;
    });
  }

  // #region character selection and construction

  private buildCurrentCharacter(currentCharacterData: any): void {
    console.log('currentCharacterData: ', currentCharacterData);
    const [
      abilitiesData,
      appearanceData,
      attributesData,
      characterData,
      combatStatsData,
      consumablesData,
      favoriteSpellsData,
      favoriteWeaponsData,
      foodData,
      hitPointsData,
      kiPointsData,
      miscellaneousData,
      moneyData,
      personalityData,
      proficienciesData,
      skillsData,
      spellLevel0,
      spellLevel1,
      spellLevel2,
      spellLevel3,
      spellLevel4,
      spellLevel5,
      spellLevel6,
      spellLevel7,
      spellLevel8,
      spellLevel9,
      spellslotsData,
      traitsData,
      weaponsAndArmorData,
    ] = currentCharacterData.map((entry: any) => entry.data);

    // currentCharacterData.forEach((entry: any, index: number) => {
    //   console.log(entry.key, ' : ', entry.data);
    // });

    console.warn('The whole character: ', currentCharacterData);

    // Attributes
    Object.keys(attributesData).forEach((key: string) => {
      this.updateAttribute(attributesData[key]);
    });

    //  Skills
    Object.keys(skillsData).forEach((key: string) => {
      this.updateSkill(skillsData[key]);
    });

    // Hit Points
    this.hitPoints = hitPointsData.hitPoints;
    this.hitDice = hitPointsData.hitDice;

    // Combat Stats
    this.armorClass = combatStatsData.armorClass;
    this.initiative = combatStatsData.initiative;
    this.movement = combatStatsData.movement;
    this.updateProficiencyBonus(combatStatsData.proficiencyBonus);
    this.deathSaves = combatStatsData.deathSaves;
    this.conditions = combatStatsData.conditions;
    this.exhaustion = combatStatsData.exhaustion;
    this.inspiration = combatStatsData.inspiration;

    // Spells
    this.spellslots = spellslotsData;
    this.favoriteSpells = favoriteSpellsData.spells;
    this.spellLevel0 = spellLevel0.spells;
    this.spellLevel1 = spellLevel1.spells;
    this.spellLevel2 = spellLevel2.spells;
    this.spellLevel3 = spellLevel3.spells;
    this.spellLevel4 = spellLevel4.spells;
    this.spellLevel5 = spellLevel5.spells;
    this.spellLevel6 = spellLevel6.spells;
    this.spellLevel7 = spellLevel7.spells;
    this.spellLevel8 = spellLevel8.spells;
    this.spellLevel9 = spellLevel9.spells;

    // Items
    this.favoriteWeapons = favoriteWeaponsData.data;
    this.weaponsAndArmor = weaponsAndArmorData.data;
    this.consumables = consumablesData.data;
    this.miscellaneous = miscellaneousData.data;
    this.food = foodData.data;
    this.money = moneyData;

    // Abilities and stuff
    this.kiPoints = kiPointsData;
    this.traits = traitsData.data;
    this.abilities = abilitiesData.data;
    this.proficiencies = proficienciesData;
  }

  // #endregion

  // #region WEAPONS

  // #endregion

  // #region # SPELLS

  private _favoriteSpells: Spell[] = [
    {
      name: 'Feuerball',
      level: 3,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: true,
      savingThrowAttribute: 'dexterity',
      school: 'evocation',
      doesDamage: true,
      damage: [
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
      ],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      isRanged: true,
      description:
        'Ein heller Strahl schießt von deinem Zeigefinger zu einem von dir gewählten Punkt in Reichweite und blüht dann mit einem tiefen Brüllen zu einer Flammenexplosion auf. Jede Kreatur in einem Umkreis von 6 Metern um diesen Punkt muss einen Rettungswurf auf Geschicklichkeit machen. Bei einem misslungenen Rettungswurf erleidet das Ziel 8W6 Feuerschaden, bei einem erfolgreichen Wurf die Hälfte des Schadens. Das Feuer breitet sich um Ecken aus. Es entzündet brennbare Gegenstände in der Umgebung, die nicht getragen werden. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      range: 150,
      hasAreaOfEffect: true,
      radius: 20,
      areaOfEffectType: 'sphere',
    },
    {
      name: 'Feuerpfeil',
      level: 0,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: true,
      needsSavingThrow: false,
      school: 'evocation',
      doesDamage: true,
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du schleuderst einen Splitter aus Feuer auf eine Kreatur in Reichweite. Lege einen Fernkampf-Zauberangriff gegen das Ziel ab. Bei einem Treffer erleidet das Ziel 1W10 Feuerschaden. Ein brennbarer Gegenstand, der von diesem Zauber getroffen wird, geht in Flammen auf, wenn er nicht getragen oder in der Hand gehalten wird. Der Schaden dieses Zaubers steigt um 1W10, wenn du die 5. Stufe (2W10), die 11. Stufe (3W10) und die 17. Stufe (4W10) erreichst.  ',
      damage: [{ diceNumber: '1', diceType: 'd10', damageType: 'fire' }],
      isRanged: true,
      range: 120,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      name: 'Vampiric Touch',
      level: 3,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsAttackRoll: true,
      needsSavingThrow: false,
      doesDamage: true,
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Die Berührung deiner schattenumrankten Hand kann anderen die Lebenskraft entziehen, um deine Wunden zu heilen. Führe einen Nahkampf-Zauberangriff gegen eine Kreatur in deiner Reichweite aus. Bei einem Treffer erleidet das Ziel 3W6 nekrotischen Schaden, und du erhältst Trefferpunkte in Höhe der Hälfte des zugefügten nekrotischen Schadens zurück. Bis der Zauber endet, kannst du den Angriff in jeder deiner Runden als Aktion wiederholen. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      school: 'necromancy',
      damage: [{ diceNumber: '3', diceType: 'd6', damageType: 'necrotic' }],
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      name: 'Heilende Berührung',
      level: 1,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: true,
      heal: { diceNumber: '1', diceType: 'd8', additionalHeal: 5 },
      description:
        'Deine Berührung kann Wunden heilen. Berühre eine Kreatur und spende ihr 1W8 + deinem Fertigkeitsbonus Trefferpunkte. Dieser Zauber hat keine Wirkung auf Untote oder Konstrukte. Wenn du diesen Zauber mit einem Zauberplatz des 2. Grades oder höher wirkst, erhöht sich die Heilung um 1W8 für jeden Grad über dem 1.',
      school: 'evocation',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
  ];

  public get favoriteSpells(): Spell[] {
    return this._favoriteSpells;
  }

  public set favoriteSpells(spells: Spell[]) {
    this._favoriteSpells = spells;
    this.setData('favoriteSpells', { spells: spells });
  }

  private _spellLevel0: Spell[] = [
    {
      name: 'Führung',
      level: 0,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: true,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du berührst eine Kreatur und bis zum Ende des Zuges erhält sie einen Bonus von 1W4 auf Angriffs- und Rettungswürfe.',
      school: 'enchantment',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      name: 'Totenläuten',
      level: 0,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: false,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: true,
      savingThrowAttribute: 'wisdom',
      doesDamage: true,
      damage: [{ diceNumber: '1', diceType: 'd8', damageType: 'necrotic' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Eine Kreatur, die du berührst, erleidet 1W8 Nekrotischen Schaden, wenn sie einen Rettungswurf auf Konstitution nicht besteht.',
      school: 'necromancy',
      isRanged: true,
      range: 60,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      name: 'Thaumaturgie',
      level: 0,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Du öffnest eine Tür, entzündest eine Kerze oder machst ein Geräusch, das du innerhalb der Reichweite deiner Stimme erzeugen kannst. Du kannst auch deine Stimme bis zum Ende deines nächsten Zuges um das Dreifache verstärken.',
      school: 'transmutation',
      isRanged: false,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
  ];

  public get spellLevel0(): Spell[] {
    return this._spellLevel0;
  }

  public set spellLevel0(spells: Spell[]) {
    this._spellLevel0 = spells;
    this.setData('spellLevel0', { spells: spells });
  }

  private _spellLevel1: Spell[] = [
    {
      name: 'Heilende Berührung',
      level: 1,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: true,
      heal: { diceNumber: '1', diceType: 'd8', additionalHeal: 5 },
      description:
        'Deine Berührung kann Wunden heilen. Berühre eine Kreatur und spende ihr 1W8 + deinem Fertigkeitsbonus Trefferpunkte. Dieser Zauber hat keine Wirkung auf Untote oder Konstrukte. Wenn du diesen Zauber mit einem Zauberplatz des 2. Grades oder höher wirkst, erhöht sich die Heilung um 1W8 für jeden Grad über dem 1.',
      school: 'evocation',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
    {
      name: 'Magie entdecken',
      level: 1,
      cost: 'action',
      canRitual: true,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsAttackRoll: false,
      needsSavingThrow: false,
      doesDamage: false,
      damage: [{ diceNumber: '', diceType: '', damageType: '' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Dein Blick wird magisch, um magische Kräfte zu erkennen. Für die Dauer erfährst du die Anzahl und die Art aller magischen Kräfte, die du innerhalb von 30 Fuß wahrnehmen kannst. Du erfährst auch die Schule der Magie, falls vorhanden.',
      school: 'divination',
      isRanged: false,
      range: 30,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
  ];

  public get spellLevel1(): Spell[] {
    return this._spellLevel1;
  }

  public set spellLevel1(spells: Spell[]) {
    this._spellLevel1 = spells;
    this.setData('spellLevel1', { spells: spells });
  }

  private _spellLevel2: Spell[] = [
    {
      name: 'Feuerball',
      level: 2,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: true,
      needsConcentration: false,
      needsSavingThrow: true,
      needsAttackRoll: false,
      savingThrowAttribute: 'dexterity',
      school: 'evocation',
      doesDamage: true,
      damage: [
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
        { diceNumber: '8', diceType: 'd6', damageType: 'fire' },
      ],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      isRanged: true,
      description:
        'Ein heller Strahl schießt von deinem Zeigefinger zu einem von dir gewählten Punkt in Reichweite und blüht dann mit einem tiefen Brüllen zu einer Flammenexplosion auf. Jede Kreatur in einem Umkreis von 6 Metern um diesen Punkt muss einen Rettungswurf auf Geschicklichkeit machen. Bei einem misslungenen Rettungswurf erleidet das Ziel 8W6 Feuerschaden, bei einem erfolgreichen Wurf die Hälfte des Schadens. Das Feuer breitet sich um Ecken aus. Es entzündet brennbare Gegenstände in der Umgebung, die nicht getragen werden. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      range: 150,
      hasAreaOfEffect: true,
      radius: 20,
      areaOfEffectType: 'sphere',
    },
    {
      name: 'Vampiric Touch',
      level: 3,
      cost: 'action',
      canRitual: false,
      needsVerbal: true,
      needsSomatic: true,
      needsMaterial: false,
      needsConcentration: true,
      needsSavingThrow: false,
      needsAttackRoll: true,
      doesDamage: true,
      damage: [{ diceNumber: '3', diceType: 'd6', damageType: 'necrotic' }],
      doesHeal: false,
      heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
      description:
        'Die Berührung deiner schattenumrankten Hand kann anderen die Lebenskraft entziehen, um deine Wunden zu heilen. Führe einen Nahkampf-Zauberangriff gegen eine Kreatur in deiner Reichweite aus. Bei einem Treffer erleidet das Ziel 3W6 nekrotischen Schaden, und du erhältst Trefferpunkte in Höhe der Hälfte des zugefügten nekrotischen Schadens zurück. Bis der Zauber endet, kannst du den Angriff in jeder deiner Runden als Aktion wiederholen. Wenn du diesen Zauber mit einem Zauberplatz des 4. Grades oder höher wirkst, erhöht sich der Schaden um 1W6 für jeden Grad über dem 3.',
      school: 'necromancy',
      isRanged: false,
      range: 5,
      hasAreaOfEffect: false,
      areaOfEffectType: '',
      radius: 0,
    },
  ];

  public get spellLevel2(): Spell[] {
    return this._spellLevel2;
  }

  public set spellLevel2(spells: Spell[]) {
    this._spellLevel2 = spells;
    this.setData('spellLevel2', { spells: spells });
  }

  private _spellLevel3: Spell[] = [];

  public get spellLevel3(): Spell[] {
    return this._spellLevel3;
  }

  public set spellLevel3(spells: Spell[]) {
    this._spellLevel3 = spells;
    this.setData('spellLevel3', { spells: spells });
  }

  private _spellLevel4: Spell[] = [];

  public get spellLevel4(): Spell[] {
    return this._spellLevel4;
  }

  public set spellLevel4(spells: Spell[]) {
    this._spellLevel4 = spells;
    this.setData('spellLevel4', { spells: spells });
  }

  private _spellLevel5: Spell[] = [];

  public get spellLevel5(): Spell[] {
    return this._spellLevel5;
  }

  public set spellLevel5(spells: Spell[]) {
    this._spellLevel5 = spells;
    this.setData('spellLevel5', { spells: spells });
  }

  private _spellLevel6: Spell[] = [];

  public get spellLevel6(): Spell[] {
    return this._spellLevel6;
  }

  public set spellLevel6(spells: Spell[]) {
    this._spellLevel6 = spells;
    this.setData('spellLevel6', { spells: spells });
  }

  private _spellLevel7: Spell[] = [];

  public get spellLevel7(): Spell[] {
    return this._spellLevel7;
  }

  public set spellLevel7(spells: Spell[]) {
    this._spellLevel7 = spells;
    this.setData('spellLevel7', { spells: spells });
  }

  private _spellLevel8: Spell[] = [];

  public get spellLevel8(): Spell[] {
    return this._spellLevel8;
  }

  public set spellLevel8(spells: Spell[]) {
    this._spellLevel8 = spells;
    this.setData('spellLevel8', { spells: spells });
  }

  private _spellLevel9: Spell[] = [];

  public get spellLevel9(): Spell[] {
    return this._spellLevel9;
  }

  public set spellLevel9(spells: Spell[]) {
    this._spellLevel9 = spells;
    this.setData('spellLevel9', { spells: spells });
  }

  public getAllPreparedSpells(): Spell[] {
    return [
      ...this._spellLevel0,
      ...this._spellLevel1,
      ...this._spellLevel2,
      ...this._spellLevel3,
      ...this._spellLevel4,
      ...this._spellLevel5,
      ...this._spellLevel6,
      ...this._spellLevel7,
      ...this._spellLevel8,
      ...this._spellLevel9,
    ];
  }

  // #endregion

  // #region ABILITIES

  private _abilities: Ability[] = [];

  public get abilities(): Ability[] {
    return this._abilities;
  }

  public set abilities(abilities: Ability[]) {
    this._abilities = abilities;
    console.warn('abilities: ', abilities);
  }

  private _traits: Trait[] = [];

  public get traits(): Trait[] {
    return this._traits;
  }

  public set traits(traits: Trait[]) {
    this._traits = traits;
    this.setData('traits', { data: traits });
  }

  private _spellslots: any = {
    spellslots: [],
    showSpellslots: true,
  };

  public get spellslots(): any {
    return this._spellslots;
  }

  public set spellslots(spellslots: any) {
    this._spellslots = spellslots;
    this.setData('spellslots', spellslots);
  }

  private _kiPoints: any = {
    totalPoints: 4,
    usedPoints: 2,
    showKiPoints: false,
  };

  public get kiPoints(): any {
    return this._kiPoints;
  }

  public set kiPoints(kiPoints: any) {
    this._kiPoints = kiPoints;
    this.setData('kiPoints', kiPoints);
  }

  private _proficiencies: any = {};

  public get proficiencies(): any {
    return this._proficiencies;
  }

  public set proficiencies(proficiencies: any) {
    this._proficiencies = proficiencies;
    this.setData('proficiencies', proficiencies);
  }

  // #endregion

  // #region # COMBAT STATS

  private proficiencySubject = new BehaviorSubject<number>(2);
  public proficiency$ = this.proficiencySubject.asObservable();

  public updateProficiencyBonus(newValue: number) {
    this.proficiencySubject.next(newValue);
    this.writeCombatStatsToDatabase();
  }

  private _armorClass: number = 10;

  public get armorClass(): number {
    return this._armorClass;
  }

  public set armorClass(newValue: number) {
    this._armorClass = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _initiative: number = 0;

  public get initiative(): number {
    return this._initiative;
  }

  public set initiative(newValue: number) {
    this._initiative = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _movement: number = 30;

  public get movement(): number {
    return this._movement;
  }

  public set movement(newValue: number) {
    this._movement = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _deathSaves: number[] = [0, 0];

  public get deathSaves(): number[] {
    return this._deathSaves;
  }

  public set deathSaves(newValue: number[]) {
    this._deathSaves = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _inspiration: boolean = false;

  public get inspiration(): boolean {
    return this._inspiration;
  }

  public set inspiration(newValue: boolean) {
    this._inspiration = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _experience: number = 0;

  public get experience(): number {
    return this._experience;
  }

  public set experience(newValue: number) {
    this._experience = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _exhaustion: number = 0;

  public get exhaustion(): number {
    return this._exhaustion;
  }

  public set exhaustion(newValue: number) {
    this._exhaustion = newValue;
    this.writeCombatStatsToDatabase();
  }

  private _conditions: string[] = [];

  public get conditions(): string[] {
    return this._conditions;
  }

  public set conditions(newValue: string[]) {
    this._conditions = newValue;
    this.writeCombatStatsToDatabase();
  }

  private writeCombatStatsToDatabase() {
    const combatStatsData = {
      armorClass: this._armorClass,
      initiative: this._initiative,
      movement: this._movement,
      deathSaves: this._deathSaves,
      proficiencyBonus: this.proficiencySubject.getValue(),
      inspiration: this._inspiration,
      experience: this._experience,
      exhaustion: this._exhaustion,
      conditions: this._conditions,
    };
    this.setData('combatStats', combatStatsData);
  }

  // #endregion

  // #region # HIT POINTS

  private _hitPoints = {
    maxHitPoints: 6,
    currentHitPoints: 6,
    temporaryHitPoints: 1,
  };

  public get hitPoints(): any {
    return this._hitPoints;
  }

  public set hitPoints(newValue: any) {
    this._hitPoints = newValue;
    this.updateHitPoints();
  }

  private _hitDice = {
    diceNumber: '1',
    diceType: 'd10',
  };

  public get hitDice(): any {
    return this._hitDice;
  }

  public set hitDice(newValue: any) {
    this._hitDice = newValue;
    this.updateHitPoints();
  }

  private updateHitPoints() {
    const hitPointsData = {
      hitPoints: this._hitPoints,
      hitDice: this._hitDice,
    };
    this.setData('hitPoints', hitPointsData);
  }

  // #endregion

  // #region # ATTRIBUTES
  updateAttribute(newValue: Attribute) {
    const functionCall: string = newValue.name + 'Subject.next(newValue)';
    eval(`this.${functionCall}`);
    const attributesData = {
      strength: this.strengthSubject.getValue(),
      dexterity: this.dexteritySubject.getValue(),
      constitution: this.constitutionSubject.getValue(),
      intelligence: this.intelligenceSubject.getValue(),
      wisdom: this.wisdomSubject.getValue(),
      charisma: this.charismaSubject.getValue(),
    };
    // TODO: ACTIVATE AGAIN
    this.setData('attributes', attributesData);
  }

  private strengthSubject = new BehaviorSubject<Attribute>(
    { name: 'strength', value: 10, proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public strength$ = this.strengthSubject.asObservable();

  private dexteritySubject = new BehaviorSubject<Attribute>(
    { name: 'dexterity', value: 10, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public dexterity$ = this.dexteritySubject.asObservable();

  private constitutionSubject = new BehaviorSubject<Attribute>(
    { name: 'constitution', value: 10, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public constitution$ = this.constitutionSubject.asObservable();

  private intelligenceSubject = new BehaviorSubject<Attribute>(
    { name: 'intelligence', value: 10, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public intelligence$ = this.intelligenceSubject.asObservable();

  private wisdomSubject = new BehaviorSubject<Attribute>(
    { name: 'wisdom', value: 10, proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public wisdom$ = this.wisdomSubject.asObservable();

  private charismaSubject = new BehaviorSubject<Attribute>(
    { name: 'charisma', value: 10, proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public charisma$ = this.charismaSubject.asObservable();

  // #endregion

  // #region # SKILLS

  updateSkill(newValue: Skill) {
    const functionCall: string = newValue.name + 'Subject.next(newValue)';
    eval(`this.${functionCall}`);
    const skillsData = {
      acrobatics: this.acrobaticsSubject.getValue(),
      animalHandling: this.animalHandlingSubject.getValue(),
      arcana: this.arcanaSubject.getValue(),
      athletics: this.athleticsSubject.getValue(),
      deception: this.deceptionSubject.getValue(),
      history: this.historySubject.getValue(),
      insight: this.insightSubject.getValue(),
      intimidation: this.intimidationSubject.getValue(),
      investigation: this.investigationSubject.getValue(),
      medicine: this.medicineSubject.getValue(),
      nature: this.natureSubject.getValue(),
      perception: this.perceptionSubject.getValue(),
      performance: this.performanceSubject.getValue(),
      persuasion: this.persuasionSubject.getValue(),
      religion: this.religionSubject.getValue(),
      sleightOfHand: this.sleightOfHandSubject.getValue(),
      stealth: this.stealthSubject.getValue(),
      survival: this.survivalSubject.getValue(),
    };
    // TODO: ACTIVATE AGAIN
    // this.setData('skills', skillsData);
  }

  private acrobaticsSubject = new BehaviorSubject<Skill>(
    { name: 'acrobatics', proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public acrobatics$ = this.acrobaticsSubject.asObservable();

  private animalHandlingSubject = new BehaviorSubject<Skill>(
    { name: 'animalHandling', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public animalHandling$ = this.animalHandlingSubject.asObservable();

  private arcanaSubject = new BehaviorSubject<Skill>(
    { name: 'arcana', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public arcana$ = this.arcanaSubject.asObservable();

  private athleticsSubject = new BehaviorSubject<Skill>(
    { name: 'athletics', proficiency: true } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public athletics$ = this.athleticsSubject.asObservable();

  private deceptionSubject = new BehaviorSubject<Skill>(
    { name: 'deception', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public deception$ = this.deceptionSubject.asObservable();

  private historySubject = new BehaviorSubject<Skill>(
    { name: 'history', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public history$ = this.historySubject.asObservable();

  private insightSubject = new BehaviorSubject<Skill>(
    { name: 'insight', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public insight$ = this.insightSubject.asObservable();

  private intimidationSubject = new BehaviorSubject<Skill>(
    { name: 'intimidation', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public intimidation$ = this.intimidationSubject.asObservable();

  private investigationSubject = new BehaviorSubject<Skill>(
    { name: 'investigation', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public investigation$ = this.investigationSubject.asObservable();

  private medicineSubject = new BehaviorSubject<Skill>(
    { name: 'medicine', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public medicine$ = this.medicineSubject.asObservable();

  private natureSubject = new BehaviorSubject<Skill>(
    { name: 'nature', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public nature$ = this.natureSubject.asObservable();

  private perceptionSubject = new BehaviorSubject<Skill>(
    { name: 'perception', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public perception$ = this.perceptionSubject.asObservable();

  private performanceSubject = new BehaviorSubject<Skill>(
    { name: 'performance', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public performance$ = this.performanceSubject.asObservable();

  private persuasionSubject = new BehaviorSubject<Skill>(
    { name: 'persuasion', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public persuasion$ = this.persuasionSubject.asObservable();

  private religionSubject = new BehaviorSubject<Skill>(
    { name: 'religion', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public religion$ = this.religionSubject.asObservable();

  private sleightOfHandSubject = new BehaviorSubject<Skill>(
    { name: 'sleightOfHand', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public sleightOfHand$ = this.sleightOfHandSubject.asObservable();

  private stealthSubject = new BehaviorSubject<Skill>(
    { name: 'stealth', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public stealth$ = this.stealthSubject.asObservable();

  private survivalSubject = new BehaviorSubject<Skill>(
    { name: 'survival', proficiency: false } //erstmal nur gemockt, später muss der Wert aus der Datenbank kommen
  );
  public survival$ = this.survivalSubject.asObservable();

  // #endregion

  // #region # inventory

  private _favoriteWeapons: Weapon[] = [];

  public get favoriteWeapons(): Weapon[] {
    return this._favoriteWeapons;
  }

  public set favoriteWeapons(weapons: Weapon[]) {
    this._favoriteWeapons = weapons;
    this.setData('favoriteWeapons', { data: this._favoriteWeapons });
  }

  private _weaponsAndArmor: SimpleItem[] = [];

  public get weaponsAndArmor(): SimpleItem[] {
    return this._weaponsAndArmor;
  }

  public set weaponsAndArmor(newValue: SimpleItem[]) {
    this._weaponsAndArmor = newValue;
    this.setData('weaponsAndArmor', { data: this._weaponsAndArmor });
  }

  private _food: Food[] = [];

  public get food(): Food[] {
    return this._food;
  }

  public set food(newValue: Food[]) {
    this._food = newValue;
    this.setData('food', { data: this._food });
  }

  private _money: any = {
    copper: 100,
    silver: 100,
    electrum: 100,
    gold: 100,
    platinum: 100,
  };

  public get money(): any {
    return this._money;
  }

  public set money(newValue: any) {
    this._money = newValue;
    this.setData('money', this._money);
  }

  private _consumbales: SimpleItem[] = [];

  public get consumables(): SimpleItem[] {
    return this._consumbales;
  }

  public set consumables(newValue: SimpleItem[]) {
    this._consumbales = newValue;
    this.setData('consumables', { data: this._consumbales });
  }

  private _miscellaneous: SimpleItem[] = [];

  public get miscellaneous(): SimpleItem[] {
    return this._miscellaneous;
  }

  public set miscellaneous(newValue: SimpleItem[]) {
    this._miscellaneous = newValue;
    this.setData('miscellaneous', { data: this._miscellaneous });
  }

  // #endregion

  // #region database calls

  public async addData(
    collection: string,
    data: any,
    key?: string
  ): Promise<void> {
    if (key) {
      return this.db
        .collection(collection)
        .add(data, key)
        .then(() => {});
    } else {
      return this.db
        .collection(collection)
        .add(data)
        .then(() => {});
    }
  }

  public getData(collection: string, key: string) {
    return this.db.collection(collection).doc(key).get();
  }

  public getCollection(collection: string) {
    return this.db.collection(collection).get();
  }

  public getCollectionWithKeys(collection: string) {
    return this.db.collection(collection).get({ keys: true });
  }

  public setData(key: string, data: any) {
    return this.db
      .collection(this.characterName)
      .doc(key)
      .set(data)
      .then(() => {});
  }

  public deleteCollection(collection: string) {
    this.db.collection(collection).delete();
  }

  // #endregion
}
