import { Injectable } from '@angular/core';
import { Spell } from 'src/interfaces/spell';
import { Subject } from 'rxjs';

@Injectable({
  providedIn: 'root',
})
export class SpellsService {
  // Functions

  private _closeSubject = new Subject<number>();

  public closeSubject$ = this._closeSubject.asObservable();

  public closeAllOthers(level: number): void {
    this._closeSubject.next(level);
  }

  // Spells
  private level0: string[] = [
    'Führung (Guidance)',
    'Totenläuten (Toll the Dead)',
    'Thaumaturgie (Thaumaturgy)',
  ];

  private level1: string[] = [
    'Heilende Berührung (Cure Wounds)',
    'Magie entdecken (Detect Magic)',
    'Segen (Bless)',
    'Heilendes Wort (Healing Word)',
    'Lenkendes Geschoss (Guiding Bolt)',
  ];

  private level2: string[] = [];

  private level3: string[] = [];

  private level4: string[] = [];

  private level5: string[] = [];

  private level6: string[] = [];

  private level7: string[] = [];

  private level8: string[] = [];

  private level9: string[] = [];

  public getListOfSpells(level: number) {
    switch (level) {
      case 0:
        return this.level0;
      case 1:
        return this.level1;
      case 2:
        return this.level2;
      case 3:
        return this.level3;
      case 4:
        return this.level4;
      case 5:
        return this.level5;
      case 6:
        return this.level6;
      case 7:
        return this.level7;
      case 8:
        return this.level8;
      case 9:
        return this.level9;
      default:
        return this.level0;
    }
  }

  public getSpell(name: string): Spell {
    switch (name) {
      case 'Führung (Guidance)':
        return this.guidance;
      case 'Totenläuten (Toll the Dead)':
        return this.tollTheDead;
      case 'Thaumaturgie (Thaumaturgy)':
        return this.thaumaturgie;
      case 'Heilende Berührung (Cure Wounds)':
        return this.cureWounds;
      case 'Magie entdecken (Detect Magic)':
        return this.detectMagic;
      case 'Segen (Bless)':
        return this.bless;
      case 'Heilendes Wort (Healing Word)':
        return this.healingWord;
      case 'Lenkendes Geschoss (Guiding Bolt)':
        return this.guidingBolt;
      default:
        return this.guidance;
    }
  }

  private guidance: Spell = {
    name: 'Führung',
    level: 0,
    cost: 'action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: true,
    needsMaterial: true,
    needsConcentration: true,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Du berührst eine Kreatur und für eine Minute erhält sie einen Bonus von 1W4 auf Angriffs- und Rettungswürfe.',
    school: 'enchantment',
    isRanged: false,
    range: 5,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private tollTheDead: Spell = {
    name: 'Totenläuten',
    level: 0,
    cost: 'action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: false,
    needsMaterial: true,
    needsConcentration: false,
    needsAttackRoll: false,
    needsSavingThrow: true,
    savingThrowAttribute: 'wisdom',
    doesDamage: true,
    damage: [{ diceNumber: '1', diceType: 'd8', damageType: 'necrotic' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Eine Kreatur, die du berührst, erleidet 1W8 Nekrotischen Schaden, wenn sie einen Rettungswurf auf Konstitution nicht besteht.',
    school: 'necromancy',
    isRanged: true,
    range: 60,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private thaumaturgie: Spell = {
    name: 'Thaumaturgie',
    level: 0,
    cost: 'action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: false,
    needsMaterial: false,
    needsConcentration: false,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Du erzeugst einen der folgenden magischen Effekte innerhalb von 9 Metern: \b\b • Deine Stimme wird dreimal lauter.\b • Du lässt einen unverschlossenen Tür oder ein Fenster aufspringen. \b• Du lässt eine Flamme aufleuchten, erlöschen oder ihre Farbe ändern. \b• Du lässt den Boden für eine Minute beben.\b • Du erzeugst Geräusche, die von dir ausgehen, wie das Flüstern einer Stimme, das Brüllen eines Löwen, das Knurren von Donner oder das Klappern von Metall. \b• Du lässt eine Tür oder ein Fenster, das du berührst, für eine Minute verriegeln oder entriegeln.',
    school: 'transmutation',
    isRanged: false,
    range: 9,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private cureWounds: Spell = {
    name: 'Heilende Berührung',
    level: 1,
    cost: 'action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: true,
    needsMaterial: false,
    needsConcentration: false,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: true,
    heal: { diceNumber: '1', diceType: 'd8', additionalHeal: 5 },
    description:
      'Deine Berührung kann Wunden heilen. Berühre eine Kreatur und spende ihr 1W8 + deinem Fertigkeitsbonus Trefferpunkte. Dieser Zauber hat keine Wirkung auf Untote oder Konstrukte. Wenn du diesen Zauber mit einem Zauberplatz des 2. Grades oder höher wirkst, erhöht sich die Heilung um 1W8 für jeden Grad über dem 1.',
    school: 'evocation',
    isRanged: false,
    range: 5,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private detectMagic: Spell = {
    name: 'Magie entdecken',
    level: 1,
    cost: 'action',
    canRitual: true,
    needsVerbal: true,
    needsSomatic: true,
    needsMaterial: true,
    needsConcentration: false,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Dein Blick wird magisch, um magische Kräfte zu erkennen. Für die Dauer erfährst du die Anzahl und die Art aller magischen Kräfte, die du innerhalb von 30 Fuß wahrnehmen kannst. Du erfährst auch die Schule der Magie, falls vorhanden.',
    school: 'divination',
    isRanged: false,
    range: 30,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private bless: Spell = {
    name: 'Segen',
    level: 1,
    cost: 'action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: true,
    needsMaterial: true,
    needsConcentration: true,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Du segnest bis zu drei Kreaturen deiner Wahl innerhalb von 9 Metern. Jedes Mal, wenn ein Ziel einen Angriffs- oder Rettungswurf macht, bevor der Zauber endet, kann das Ziel einen Würfelwurf von 1W4 auf den Wurf machen und den Wurf um die gewürfelte Zahl erhöhen.',
    school: 'enchantment',
    isRanged: false,
    range: 9,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private healingWord: Spell = {
    name: 'Heilendes Wort',
    level: 1,
    cost: 'bonus action',
    canRitual: false,
    needsVerbal: true,
    needsSomatic: false,
    needsMaterial: false,
    needsConcentration: false,
    needsAttackRoll: false,
    needsSavingThrow: false,
    doesDamage: false,
    damage: [{ diceNumber: '', diceType: '', damageType: '' }],
    doesHeal: true,
    heal: { diceNumber: '1', diceType: 'd4', additionalHeal: 0 },
    description:
      'Ein Wort der Heilung flüstert von dir und berührt bis zu sechs Kreaturen deiner Wahl innerhalb von 18 Metern. Jede Kreatur erhält 1W4 + deinen Fertigkeitsbonus Trefferpunkte.',
    school: 'evocation',
    isRanged: true,
    range: 90,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };

  private guidingBolt: Spell = {
    name: 'Lenkendes Geschoss',
    level: 1,
    cost: 'action',
    canRitual: false,
    needsVerbal: false,
    needsSomatic: true,
    needsMaterial: false,
    needsConcentration: false,
    needsAttackRoll: true,
    needsSavingThrow: false,
    doesDamage: true,
    damage: [{ diceNumber: '3', diceType: 'd4', damageType: 'force' }],
    doesHeal: false,
    heal: { diceNumber: '', diceType: '', additionalHeal: 0 },
    description:
      'Du schießt drei magische Geschosse auf Kreaturen oder Objekte innerhalb von 30 Fuß. Du kannst sie auf ein Ziel oder mehrere verteilen. Du kannst auch mehrere Geschosse auf ein Ziel abfeuern. Mache für jedes Geschoss einen Angriffswurf. Ein Geschoss trifft und verursacht 1W4 + 1 Kraftschaden.',
    school: 'evocation',
    isRanged: true,
    range: 30,
    hasAreaOfEffect: false,
    areaOfEffectType: '',
    radius: 0,
  };
}
